@@ -23,27 +23,11 @@ import silver_platter  # noqa: F401
 import argparse
 import sys
 
-from .changer import (
-    setup_parser_common,
-    run_single_changer,
-    changer_subcommands,
-    changer_subcommand,
-)
-
-from . import uploader as debian_uploader
-
-
-def run_changer_subcommand(name, changer_cls, argv):
-    parser = argparse.ArgumentParser(prog="debian-svp %s URL|package" % name)
-    setup_parser_common(parser)
-    parser.add_argument("package", type=str, nargs="?")
-    changer_cls.setup_parser(parser)
-    args = parser.parse_args(argv)
-    if args.package is None:
-        parser.print_usage()
-        return 1
-    changer = changer_cls.from_args(args)
-    return run_single_changer(changer, args)
+from . import (
+    apply as debian_apply,
+    run as debian_run,
+    uploader as debian_uploader,
+    )
 
 
 def main(argv: Optional[List[str]] = None) -> Optional[int]:
@@ -55,6 +39,8 @@ def main(argv: Optional[List[str]] = None) -> Optional[int]:
 
     subcommands: Dict[str, Callable[[List[str]], Optional[int]]] = {
         "upload-pending": debian_uploader.main,
+        "apply": debian_apply.main,
+        "run": debian_run.main,
     }
 
     parser = argparse.ArgumentParser(prog="debian-svp", add_help=False)
@@ -63,17 +49,20 @@ def main(argv: Optional[List[str]] = None) -> Optional[int]:
         action="version",
         version="%(prog)s " + silver_platter.version_string,
     )
+    parser.add_argument(
+        "--debug",
+        action="store_true",
+        help="Be more verbose")
     parser.add_argument(
         "--help", action="store_true", help="show this help message and exit"
     )
 
-    subcommands.update(main_subcommands.items())
-
-    # We have a debian-specific run command
-    del subcommands["run"]
+    for name, cmd in main_subcommands.items():
+        if name not in subcommands:
+            subcommands[name] = cmd
 
     parser.add_argument(
-        "subcommand", type=str, choices=list(subcommands.keys()) + changer_subcommands()
+        "subcommand", type=str, choices=list(subcommands.keys())
     )
     args, rest = parser.parse_known_args()
     if args.help:
@@ -82,19 +71,16 @@ def main(argv: Optional[List[str]] = None) -> Optional[int]:
             parser.exit()
         else:
             rest.append("--help")
-
+    if args.debug:
+        level = logging.DEBUG
+    else:
+        level = logging.INFO
+    logging.basicConfig(level=level, format="%(message)s")
     if args.subcommand is None:
         parser.print_usage()
         return 1
     if args.subcommand in subcommands:
         return subcommands[args.subcommand](rest)
-    try:
-        subcmd = changer_subcommand(args.subcommand)
-    except KeyError:
-        pass
-    else:
-        logging.basicConfig(level=logging.INFO)
-        return run_changer_subcommand(args.subcommand, subcmd, rest)
     parser.print_usage()
     return 1
 
