@@ -30,6 +30,7 @@ from typing import List
 
 from debmutate.changelog import (
     ChangelogEditor,
+    ChangelogParseError,
     changeblock_ensure_first_line,
 )
 from debmutate.control import ControlEditor
@@ -50,9 +51,10 @@ from breezy.plugins.debian.util import (
     changelog_find_previous_upload,
     dput_changes,
     find_changelog,
-    debsign,
     MissingChangelogError,
+    NoPreviousUpload,
 )
+from breezy.plugins.debian.upstream import MissingUpstreamTarball
 
 from debian.changelog import get_maintainer
 
@@ -70,9 +72,30 @@ from ..utils import (
     BranchUnavailable,
     BranchMissing,
     BranchUnsupported,
+    BranchRateLimited,
 )
 
 
+def connect_udd_mirror():
+    import psycopg2
+
+    return psycopg2.connect(
+        database="udd",
+        user="udd-mirror",
+        password="udd-mirror",
+        host="udd-mirror.debian.net",
+    )
+
+
+def debsign(path, keyid=None):
+    (bd, changes_file) = os.path.split(path)
+    args = ["debsign"]
+    if keyid:
+        args.append("-k%s" % keyid)
+    args.append(changes_file)
+    subprocess.check_call(args, cwd=bd)
+
+
 class LastUploadMoreRecent(Exception):
     """Last version in archive is newer than vcs version."""
 
@@ -174,7 +197,11 @@ def get_maintainer_keys(context):
             yield subkey.keyid
 
 
-def prepare_upload_package(
+class GbpDchFailed(Exception):
+    """gbp dch failed to run"""
+
+
+def prepare_upload_package(  # noqa: C901
     local_tree,
     subpath,
     pkg,
@@ -185,15 +212,24 @@ def prepare_upload_package(
     allowed_committers=None,
 ):
     if local_tree.has_filename(os.path.join(subpath, "debian/gbp.conf")):
-        subprocess.check_call(
-            ["gbp", "dch", "--ignore-branch"], cwd=local_tree.abspath(".")
-        )
+        try:
+            subprocess.check_call(
+                ["gbp", "dch", "--ignore-branch"], cwd=local_tree.abspath(".")
+            )
+        except subprocess.CalledProcessError:
+            # TODO(jelmer): gbp dch sometimes fails when there is no existing
+            # open changelog entry; it fails invoking "dpkg --lt None <old-version>"
+            raise GbpDchFailed()
     cl, top_level = find_changelog(local_tree, merge=False, max_blocks=None)
     if cl.version == last_uploaded_version:
         raise NoUnuploadedChanges(cl.version)
-    previous_version_in_branch = changelog_find_previous_upload(cl)
-    if last_uploaded_version > previous_version_in_branch:
-        raise LastUploadMoreRecent(last_uploaded_version, previous_version_in_branch)
+    try:
+        previous_version_in_branch = changelog_find_previous_upload(cl)
+    except NoPreviousUpload:
+        pass
+    else:
+        if last_uploaded_version > previous_version_in_branch:
+            raise LastUploadMoreRecent(last_uploaded_version, previous_version_in_branch)
 
     logging.info("Checking revisions since %s" % last_uploaded_version)
     with local_tree.lock_read():
@@ -254,8 +290,8 @@ def prepare_upload_package(
     target_changes = _build_helper(
         local_tree, subpath, local_tree.branch, target_dir, builder=builder
     )
-    debsign(target_changes)
-    return target_changes, tag_name
+    debsign(target_changes['source'])
+    return target_changes['source'], tag_name
 
 
 def select_apt_packages(package_names, maintainer):
@@ -281,14 +317,7 @@ def select_apt_packages(package_names, maintainer):
 def select_vcswatch_packages(
     packages: List[str], maintainer: List[str], autopkgtest_only: bool
 ):
-    import psycopg2
-
-    conn = psycopg2.connect(
-        database="udd",
-        user="udd-mirror",
-        password="udd-mirror",
-        host="udd-mirror.debian.net",
-    )
+    conn = connect_udd_mirror()
     cursor = conn.cursor()
     args = []
     query = """\
@@ -301,10 +330,10 @@ def select_vcswatch_packages(
     if autopkgtest_only:
         query += " AND sources.testsuite != '' "
     if maintainer:
-        query += " AND sources.maintainer_email IN (%s)"
+        query += " AND sources.maintainer_email in %s"
         args.append(tuple(maintainer))
     if packages:
-        query += " AND sources.source IN (%s)"
+        query += " AND sources.source IN %s"
         args.append(tuple(packages))
 
     cursor.execute(query, tuple(args))
@@ -377,6 +406,7 @@ def main(argv):  # noqa: C901
     )
 
     args = parser.parse_args(argv)
+
     ret = 0
 
     if not args.packages and not args.maintainer:
@@ -410,6 +440,26 @@ def main(argv):  # noqa: C901
     # TODO(jelmer): Sort packages by last commit date; least recently changed
     # commits are more likely to be successful.
 
+    stats = {
+        'not-in-apt': 0,
+        'not-in-vcs': 0,
+        'vcs-inaccessible': 0,
+        'gbp-dch-failed': 0,
+        'missing-upstream-tarball': 0,
+        'committer-not-allowed': 0,
+        'build-failed': 0,
+        'last-release-missing': 0,
+        'last-upload-not-in-vcs': 0,
+        'missing-changelog': 0,
+        'recent-commits': 0,
+        'no-unuploaded-changes': 0,
+        'no-unreleased-changes': 0,
+        'vcs-permission-denied': 0,
+        'changelog-parse-error': 0,
+        }
+    if args.autopkgtest_only:
+        stats['no-autopkgtest'] = 0
+
     if len(packages) > 1:
         logging.info("Uploading packages: %s", ", ".join(packages))
 
@@ -421,13 +471,17 @@ def main(argv):  # noqa: C901
             try:
                 pkg_source = apt_get_source_package(package)
             except NoSuchPackage:
+                stats['not-in-apt'] += 1
                 logging.info("%s: package not found in apt", package)
                 ret = 1
                 continue
             try:
                 vcs_type, vcs_url = source_package_vcs(pkg_source)
             except KeyError:
-                logging.info("%s: no declared vcs location, skipping", pkg_source["Package"])
+                stats['not-in-vcs'] += 1
+                logging.info(
+                    "%s: no declared vcs location, skipping", pkg_source["Package"]
+                )
                 ret = 1
                 continue
             source_name = pkg_source["Package"]
@@ -448,6 +502,7 @@ def main(argv):  # noqa: C901
         try:
             main_branch = open_branch(location, probers=probers, name=branch_name)
         except (BranchUnavailable, BranchMissing, BranchUnsupported) as e:
+            stats['vcs-inaccessible'] += 1
             logging.exception("%s: %s", vcs_url, e)
             ret = 1
             continue
@@ -471,8 +526,8 @@ def main(argv):  # noqa: C901
                     os.path.join(subpath, "debian/tests/control")
                 )
             ):
-                logging.info(
-                    "%s: Skipping, package has no autopkgtest.", source_name)
+                logging.info("%s: Skipping, package has no autopkgtest.", source_name)
+                stats['no-autopkgtest'] += 1
                 continue
             branch_config = ws.local_tree.branch.get_config_stack()
             if args.gpg_verification:
@@ -496,7 +551,23 @@ def main(argv):  # noqa: C901
                     min_commit_age=args.min_commit_age,
                     allowed_committers=args.allowed_committer,
                 )
+            except GbpDchFailed as e:
+                logging.warn("%s: 'gbp dch' failed to run: %s", source_name, e)
+                stats['gbp-dch-failed'] += 1
+                continue
+            except MissingUpstreamTarball as e:
+                stats['missing-upstream-tarball'] += 1
+                logging.warning("%s: missing upstream tarball: %s", source_name, e)
+                continue
+            except BranchRateLimited as e:
+                stats['rate-limited'] += 1
+                logging.warning(
+                    '%s: rate limited by server (retrying after %s)',
+                    source_name, e.retry_after)
+                ret = 1
+                continue
             except CommitterNotAllowed as e:
+                stats['committer-not-allowed'] += 1
                 logging.warn(
                     "%s: committer %s not in allowed list: %r",
                     source_name,
@@ -506,6 +577,7 @@ def main(argv):  # noqa: C901
                 continue
             except BuildFailedError as e:
                 logging.warn("%s: package failed to build: %s", source_name, e)
+                stats['build-failed'] += 1
                 ret = 1
                 continue
             except LastReleaseRevisionNotFound as e:
@@ -515,9 +587,11 @@ def main(argv):  # noqa: C901
                     source_name,
                     e.version,
                 )
+                stats['last-release-missing'] += 1
                 ret = 1
                 continue
             except LastUploadMoreRecent as e:
+                stats['last-upload-not-in-vcs'] += 1
                 logging.warn(
                     "%s: Last upload (%s) was more recent than VCS (%s)",
                     source_name,
@@ -526,23 +600,29 @@ def main(argv):  # noqa: C901
                 )
                 ret = 1
                 continue
+            except ChangelogParseError as e:
+                stats['changelog-parse-error'] += 1
+                logging.info("%s: Error parsing changelog: %s", source_name, e)
+                ret = 1
+                continue
             except MissingChangelogError:
+                stats['missing-changelog'] += 1
                 logging.info("%s: No changelog found, skipping.", source_name)
                 ret = 1
                 continue
             except RecentCommits as e:
+                stats['recent-commits'] += 1
                 logging.info(
-                    "%s: Recent commits (%d days), skipping.",
-                    source_name, e.commit_age
+                    "%s: Recent commits (%d days), skipping.", source_name, e.commit_age
                 )
                 continue
             except NoUnuploadedChanges:
-                logging.info(
-                    "%s: No unuploaded changes, skipping.", source_name)
+                stats['no-unuploaded-changes'] += 1
+                logging.info("%s: No unuploaded changes, skipping.", source_name)
                 continue
             except NoUnreleasedChanges:
-                logging.info(
-                    "%s: No unreleased changes, skipping.", source_name)
+                stats['no-unreleased-changes'] += 1
+                logging.info("%s: No unreleased changes, skipping.", source_name)
                 continue
 
             tags = []
@@ -552,9 +632,10 @@ def main(argv):  # noqa: C901
             try:
                 ws.push(dry_run=args.dry_run, tags=tags)
             except PermissionDenied:
+                stats['vcs-permission-denied'] += 1
                 logging.info(
-                    "%s: Permission denied pushing to branch, skipping.",
-                    source_name)
+                    "%s: Permission denied pushing to branch, skipping.", source_name
+                )
                 ret = 1
                 continue
             if not args.dry_run:
@@ -564,6 +645,11 @@ def main(argv):  # noqa: C901
                 ws.show_diff(sys.stdout.buffer)
                 sys.stdout.buffer.flush()
 
+    if len(packages) > 1:
+        logging.info('Results:')
+        for error, c in stats.items():
+            logging.info('  %s: %d', error, c)
+
     return ret
 
 
