@@ -41,19 +41,10 @@ from breezy.propose import (
     HosterLoginRequired,
 )
 
-try:
-    from breezy.propose import (
-        iter_hoster_instances,
-        SourceNotDerivedFromTarget,
-    )
-except ImportError:  # breezy < 3.1.1
-
-    def iter_hoster_instances():
-        for name, hoster_cls in hosters.items():
-            for instance in hoster_cls.iter_instances():
-                yield instance
-
-    SourceNotDerivedFromTarget = None
+from breezy.propose import (
+    iter_hoster_instances,
+    SourceNotDerivedFromTarget,
+)
 
 import breezy.plugins.gitlab  # noqa: F401
 import breezy.plugins.github  # noqa: F401
@@ -74,10 +65,6 @@ from .publish import (
     find_existing_proposed,
     SUPPORTED_MODES,
 )
-from .workspace import (
-    Workspace,
-    PublishResult,
-)
 
 
 __all__ = [
@@ -92,8 +79,6 @@ __all__ = [
     "SUPPORTED_MODES",
     "push_derived_changes",
     "propose_changes",
-    "PublishResult",
-    "Workspace",
     "DryRunProposal",
     "check_proposal_diff",
     "EmptyMergeProposal",
@@ -178,8 +163,8 @@ def iter_conflicted(
         resume_branch = open_branch(
             mp.get_source_branch_url(), possible_transports=possible_transports
         )
-        if resume_branch.name != branch_name and not (
-            not resume_branch.name and resume_branch.user_url.endswith(branch_name)
+        if resume_branch.name != branch_name and not (  # type: ignore
+            not resume_branch.name and resume_branch.user_url.endswith(branch_name)  # type: ignore
         ):
             continue
         # TODO(jelmer): Find out somehow whether we need to modify a subpath?
