@@ -25,6 +25,7 @@ from breezy import (
     revision as _mod_revision,
 )
 from breezy.errors import PermissionDenied
+from breezy.memorybranch import MemoryBranch
 from breezy.propose import (
     get_hoster,
     Hoster,
@@ -35,17 +36,13 @@ from breezy.propose import (
 )
 from breezy.transport import Transport
 
-try:
-    from breezy.propose import (
-        SourceNotDerivedFromTarget,
-    )
-except ImportError:  # breezy < 3.1.1
-    SourceNotDerivedFromTarget = None
+from breezy.propose import (
+    SourceNotDerivedFromTarget,
+)
 
 
 from .utils import (
     open_branch,
-    MemoryBranch,
     full_branch_url,
 )
 
@@ -61,6 +58,7 @@ __all__ = [
     "NoSuchProject",
     "PermissionDenied",
     "UnsupportedHoster",
+    "SourceNotDerivedFromTarget",
 ]
 
 
@@ -84,7 +82,7 @@ def _tag_selector_from_tags(tags):
 def push_result(
     local_branch: Branch,
     remote_branch: Branch,
-    additional_colocated_branches: Optional[List[str]] = None,
+    additional_colocated_branches: Optional[Union[List[str], Dict[str, str]]] = None,
     tags: Optional[Union[Dict[str, bytes], List[str]]] = None,
     stop_revision: Optional[bytes] = None,
 ) -> None:
@@ -98,13 +96,17 @@ def push_result(
     except errors.LockFailed as e:
         # Almost certainly actually a PermissionDenied error..
         raise errors.PermissionDenied(path=full_branch_url(remote_branch), extra=e)
-    for branch_name in additional_colocated_branches or []:
+    for from_branch_name in additional_colocated_branches or []:
         try:
-            add_branch = local_branch.controldir.open_branch(name=branch_name)
+            add_branch = local_branch.controldir.open_branch(name=from_branch_name)  # type: ignore
         except errors.NotBranchError:
             pass
         else:
-            remote_branch.controldir.push_branch(add_branch, name=branch_name, **kwargs)
+            if isinstance(additional_colocated_branches, dict):
+                to_branch_name = additional_colocated_branches[from_branch_name]
+            else:
+                to_branch_name = from_branch_name
+            remote_branch.controldir.push_branch(add_branch, name=to_branch_name, **kwargs)  # type: ignore
 
 
 def push_changes(
@@ -112,7 +114,7 @@ def push_changes(
     main_branch: Branch,
     hoster: Optional[Hoster],
     possible_transports: Optional[List[Transport]] = None,
-    additional_colocated_branches: Optional[List[str]] = None,
+    additional_colocated_branches: Optional[Union[List[str], Dict[str, str]]] = None,
     dry_run: bool = False,
     tags: Optional[Union[Dict[str, bytes], List[str]]] = None,
     stop_revision: Optional[bytes] = None,
@@ -171,7 +173,7 @@ def propose_changes(  # noqa: C901
     labels: Optional[List[str]] = None,
     dry_run: bool = False,
     commit_message: Optional[str] = None,
-    additional_colocated_branches: Optional[List[str]] = None,
+    additional_colocated_branches: Optional[Union[List[str], Dict[str, str]]] = None,
     allow_empty: bool = False,
     reviewers: Optional[List[str]] = None,
     tags: Optional[Union[Dict[str, bytes], List[str]]] = None,
@@ -226,18 +228,22 @@ def propose_changes(  # noqa: C901
                 owner=owner,
                 **push_kwargs
             )
-        for colocated_branch_name in additional_colocated_branches or []:
+        for from_branch_name in additional_colocated_branches or []:
             try:
-                local_colo_branch = local_branch.controldir.open_branch(
-                    name=colocated_branch_name
+                local_colo_branch = local_branch.controldir.open_branch(  # type: ignore
+                    name=from_branch_name
                 )
             except errors.NotBranchError:
                 pass
             else:
-                remote_branch.controldir.push_branch(
+                if isinstance(additional_colocated_branches, dict):
+                    to_branch_name = additional_colocated_branches[from_branch_name]
+                else:
+                    to_branch_name = from_branch_name
+                remote_branch.controldir.push_branch(  # type: ignore
                     source=local_colo_branch,
                     overwrite=overwrite_existing,
-                    name=colocated_branch_name,
+                    name=to_branch_name,
                     **push_kwargs
                 )
     if resume_proposal is not None and dry_run:
@@ -254,10 +260,9 @@ def propose_changes(  # noqa: C901
         )
 
         try:
-            resume_proposal.reopen()
+            resume_proposal.reopen()  # type: ignore
         except ReopenFailed:
-            logging.info(
-                "Reopening existing proposal failed. Creating new proposal.")
+            logging.info("Reopening existing proposal failed. Creating new proposal.")
             resume_proposal = None
     if resume_proposal is None:
         if not dry_run:
@@ -278,8 +283,7 @@ def propose_changes(  # noqa: C901
                     raise
                 resume_proposal = e.existing_proposal
             except errors.PermissionDenied:
-                logging.info(
-                    "Permission denied while trying to create " "proposal.")
+                logging.info("Permission denied while trying to create " "proposal.")
                 raise
             else:
                 return (mp, True)
@@ -335,7 +339,7 @@ def check_proposal_diff(
             merger.find_base()
         except errors.UnrelatedBranches:
             merger.set_base_revision(_mod_revision.NULL_REVISION, other_branch)
-        merger.merge_type = _mod_merge.Merge3Merger
+        merger.merge_type = _mod_merge.Merge3Merger  # type: ignore
         tree_merger = merger.make_merger()
         with tree_merger.make_preview_transform() as tt:
             changes = tt.iter_changes()
@@ -435,6 +439,7 @@ def find_existing_proposed(
     name: str,
     overwrite_unrelated: bool = False,
     owner: Optional[str] = None,
+    preferred_schemes: Optional[List[str]] = None,
 ) -> Tuple[Optional[Branch], Optional[bool], Optional[MergeProposal]]:
     """Find an existing derived branch with the specified name, and proposal.
 
@@ -444,13 +449,21 @@ def find_existing_proposed(
       name: Name of the derived branch
       overwrite_unrelated: Whether to overwrite existing (but unrelated)
         branches
+      preferred_schemes: List of preferred schemes
     Returns:
       Tuple with (resume_branch, overwrite_existing, existing_proposal)
       The resume_branch is the branch to continue from; overwrite_existing
       means there is an existing branch in place that should be overwritten.
     """
     try:
-        existing_branch = hoster.get_derived_branch(main_branch, name=name, owner=owner)
+        if preferred_schemes is not None:
+            existing_branch = hoster.get_derived_branch(
+                main_branch, name=name, owner=owner, preferred_schemes=preferred_schemes
+            )
+        else:  # TODO: Support older versions of breezy without preferred_schemes
+            existing_branch = hoster.get_derived_branch(
+                main_branch, name=name, owner=owner
+            )
     except errors.NotBranchError:
         return (None, None, None)
     else:
@@ -548,6 +561,10 @@ class PublishResult(object):
         return (self.proposal, self.is_new)
 
 
+class InsufficientChangesForNewProposal(Exception):
+    """There were not enough changes for a new merge proposal."""
+
+
 def publish_changes(
     local_branch: Branch,
     main_branch: Branch,
@@ -651,8 +668,7 @@ def publish_changes(
 
     assert mode == "propose"
     if not resume_branch and not allow_create_proposal:
-        # TODO(jelmer): Raise an exception of some sort here?
-        return PublishResult(mode)
+        raise InsufficientChangesForNewProposal()
 
     mp_description = get_proposal_description(
         getattr(hoster, "merge_proposal_description_format", "plain"),
