@@ -20,11 +20,11 @@ import unittest
 
 def test_suite():
     names = [
+        "candidates",
         "debian",
-        "debian_lintian",
-        "debian_upstream",
         "proposal",
         "publish",
+        "recipe",
         "run",
         "utils",
         "version",
