@@ -17,7 +17,7 @@
 
 from breezy.tests import (
     TestCaseWithTransport,
-    )
+)
 
 from ..utils import (
     TemporarySprout,
@@ -25,73 +25,78 @@ from ..utils import (
     run_post_check,
     PreCheckFailed,
     PostCheckFailed,
-    )
+)
 
 
 class TemporarySproutTests(TestCaseWithTransport):
-
     def test_simple(self):
-        builder = self.make_branch_builder('.')
+        builder = self.make_branch_builder(".")
         builder.start_series()
-        orig_revid = builder.build_snapshot(None, [
-            ('add', ('', None, 'directory', '')),
-            ('add', ('debian/', None, 'directory', '')),
-            ('add', ('debian/control', None, 'file', b'initial'))],
-            message='Initial\n')
+        orig_revid = builder.build_snapshot(
+            None,
+            [
+                ("add", ("", None, "directory", "")),
+                ("add", ("debian/", None, "directory", "")),
+                ("add", ("debian/control", None, "file", b"initial")),
+            ],
+            message="Initial\n",
+        )
         builder.finish_series()
         branch = builder.get_branch()
         with TemporarySprout(branch, dir=self.test_dir) as tree:
             self.assertNotEqual(branch.control_url, tree.branch.control_url)
-            tree.commit('blah')
+            tree.commit("blah")
             # Commits in the temporary sprout don't affect the original branch.
             self.assertEqual(branch.last_revision(), orig_revid)
 
     def test_nonexistent_colocated(self):
         # Colocated branches that are specified but don't exist are ignored.
-        builder = self.make_branch_builder('.')
+        builder = self.make_branch_builder(".")
         builder.start_series()
-        orig_revid = builder.build_snapshot(None, [
-            ('add', ('', None, 'directory', '')),
-            ('add', ('debian/', None, 'directory', ''))],
-            message='Initial\n')
+        orig_revid = builder.build_snapshot(
+            None,
+            [
+                ("add", ("", None, "directory", "")),
+                ("add", ("debian/", None, "directory", "")),
+            ],
+            message="Initial\n",
+        )
         builder.finish_series()
         branch = builder.get_branch()
-        with TemporarySprout(branch, ['foo'], dir=self.test_dir) as tree:
+        with TemporarySprout(branch, ["foo"], dir=self.test_dir) as tree:
             self.assertNotEqual(branch.control_url, tree.branch.control_url)
-            tree.commit('blah')
+            tree.commit("blah")
             # Commits in the temporary sprout don't affect the original branch.
             self.assertEqual(branch.last_revision(), orig_revid)
 
 
 class RunPreCheckTests(TestCaseWithTransport):
-
     def test_none(self):
-        tree = self.make_branch_and_tree('tree')
+        tree = self.make_branch_and_tree("tree")
         self.assertIs(run_pre_check(tree, None), None)
 
     def test_false(self):
-        tree = self.make_branch_and_tree('tree')
+        tree = self.make_branch_and_tree("tree")
         self.assertRaises(PreCheckFailed, run_pre_check, tree, "/bin/false")
 
     def test_true(self):
-        tree = self.make_branch_and_tree('tree')
+        tree = self.make_branch_and_tree("tree")
         self.assertIs(run_pre_check(tree, "/bin/true"), None)
 
 
 class RunPostCheckTests(TestCaseWithTransport):
-
     def test_none(self):
-        tree = self.make_branch_and_tree('tree')
+        tree = self.make_branch_and_tree("tree")
         self.assertIs(run_post_check(tree, None, None), None)
 
     def test_false(self):
-        tree = self.make_branch_and_tree('tree')
-        cid = tree.commit('a')
+        tree = self.make_branch_and_tree("tree")
+        cid = tree.commit("a")
         self.assertRaises(
-            PostCheckFailed, run_post_check, tree, "/bin/false",
-            since_revid=cid)
+            PostCheckFailed, run_post_check, tree, "/bin/false", since_revid=cid
+        )
 
     def test_true(self):
-        tree = self.make_branch_and_tree('tree')
-        cid = tree.commit('a')
+        tree = self.make_branch_and_tree("tree")
+        cid = tree.commit("a")
         self.assertIs(run_post_check(tree, "/bin/true", since_revid=cid), None)
